/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */
import type { DatabasePath } from "../interfaces/databasePath.js";

const validators: Record<DatabasePath, Array<string>> = {
  appeals: [
    "firstName",
    "lastName",
    "email",
    "understandBinding",
    "sanctionType",
    "caseNumber",
    "sanctionPlatform",
    "platformUsername",
    "sanctionReason",
    "sanctionFair",
    "behaviourViolation",
    "appealReason",
    "behaviourImprove",
  ],
  commissions: [ "firstName", "lastName", "email", "companyName", "request" ],
  contacts:    [ "firstName", "lastName", "email", "companyName", "request" ],
  events:      [
    "firstName",
    "lastName",
    "email",
    "companyName",
    "eventDescription",
    "eventTopic",
    "eventLocation",
    "eventDate",
    "eventBudget",
    "travelCovered",
    "lodgingCovered",
    "foodCovered",
  ],
  meetings: [
    "firstName",
    "lastName",
    "email",
    "companyName",
    "sessionLength",
    "sessionGoal",
    "paymentUnderstanding",
  ],
  mentorships: [
    "firstName",
    "lastName",
    "email",
    "companyName",
    "mentorshipGoal",
    "currentFocus",
    "paymentUnderstanding",
  ],
  staff: [
    "firstName",
    "lastName",
    "email",
    "understandVolunteer",
    "platform",
    "platformUsername",
    "whyJoin",
    "currentBehaviour",
    "priorExperience",
    "internalConflict",
    "handlingTrauma",
    "difficultSituation",
    "leadershipSituation",
  ],
};

/**
 * Confirms that all expected properties are present in the body.
 * Asserts that the user has consented to receiving communications.
 * Validates that values are not falsy.
 * @param body - The request body.
 * @param type - The type of form being submitted.
 * @returns A string if the body is invalid, or null if it is valid.
 */
// eslint-disable-next-line complexity -- No.
export const validateBody = (
  body: Record<string, unknown>,
  type: DatabasePath,
): string | null => {
  if (!("consent" in body) || body.consent !== true) {
    // eslint-disable-next-line stylistic/max-len -- This is a long string.
    return "You must consent to receiving communications in order to submit our forms.";
  }
  for (const key of validators[type]) {
    if (!(key in body) || body[key] === undefined || body[key] === null) {
      return `The ${key} field is required.`;
    }
    if (typeof body[key] === "string" && body[key].trim() === "") {
      return `The ${key} field cannot be empty.`;
    }
    if (typeof body[key] === "number" && Number.isNaN(body[key])) {
      return `The ${key} field must be a number.`;
    }
    if (typeof body[key] === "boolean" && !body[key]) {
      return `The ${key} field must be true.`;
    }
    if (Array.isArray(body[key]) && body[key].length === 0) {
      return `The ${key} field must have at least one item.`;
    }
    if (typeof body[key] === "object" && Object.keys(body[key]).length === 0) {
      return `The ${key} field must have at least one property.`;
    }
  }
  return null;
};
